// Relay.cpp : Defines the entry point for the DLL application.
//


#include "bzfsAPI.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <iostream>
#include <cstdlib>
#include <fstream>

BZ_GET_PLUGIN_VERSION


std::string nick = "BZRelay"; // put the "bot"'s nick
std::string channel = "#main"; // the channel that it is going to join

int fd; 
void* respondPing(void*);

class RelayHandler : public bz_EventHandler
{
public:
	RelayHandler();
	virtual ~RelayHandler();

	virtual void process ( bz_EventData *eventData );

	virtual bool autoDelete ( void ) { return false;} // this will be used for more then one event

	

};
RelayHandler relay;

BZF_PLUGIN_CALL int bz_Load ( const char* commandLine )
{
	bz_debugMessage(1,"Relay plugin loaded");
	
	bz_registerEvent ( bz_eChatMessageEvent, &relay );
	bz_registerEvent ( bz_ePlayerJoinEvent, &relay );
	bz_registerEvent ( bz_ePlayerPartEvent, &relay );

	return 0;
}

BZF_PLUGIN_CALL int bz_Unload ( void )
{
	bz_debugMessage(1,"Relay plugin unloaded");

	bz_removeEvent ( bz_eChatMessageEvent, &relay );
	bz_removeEvent ( bz_ePlayerJoinEvent, &relay );
	bz_removeEvent ( bz_ePlayerPartEvent, &relay );

	return 0;
}
RelayHandler::RelayHandler()
{

	fd = socket(AF_INET, SOCK_STREAM, 0);	
	struct sockaddr_in dest_addr;
	dest_addr.sin_family = AF_INET;

	std::ifstream x("config.txt");
	char addressc[256], channelc[256], nickc[256], buffer[256];
	for(int i = 0; i < 6; i++)
	x.getline (buffer,256);
	
	x.getline (addressc,256);
	x.getline (nickc,256);
	x.getline (channelc,256);

	x.close();
	nick = nickc;
	channel = channelc;
	std::string address = addressc;

	dest_addr.sin_addr.s_addr = inet_addr(address.c_str());//address of irc server
	dest_addr.sin_port = htons(6667);//port of irc server

	memset(&(dest_addr.sin_zero), '\0', 8);

	
	if(connect(fd, (struct sockaddr *)&dest_addr, sizeof(struct sockaddr))!=0){
	bz_debugMessage(1,"Connecting to irc server");
	}
	else{
	bz_debugMessage(1,"ERROR! Could not connect!");
	}
	

	char recv_buf[1024];
	int r_len;
	

	std::string str1 = "NICK " + nick + "\r\n";
	std::string str2 = "USER foo google.com google.com " + nick + "\r\n";
	std::string str3 = "JOIN " + channel + "\r\n";
	
	
	// recieve something before sending
	r_len = read(fd, recv_buf, 1024);
	recv_buf[r_len] = '\0';
	std::string message = recv_buf;

	// send username, join channel
	write(fd, str1.c_str(), str1.size());
	write(fd, str2.c_str(), str2.size());
	write(fd, str3.c_str(), str3.size());

	pthread_t thread; 
	pthread_create(&thread,NULL,respondPing,NULL); // threading... run a func that'll process irc input
}

RelayHandler::~RelayHandler()
{
}

void RelayHandler::process ( bz_EventData *eventData )
{
	switch (eventData->eventType)
	{
		case bz_eChatMessageEvent:
		{
			
			bz_ChatEventData *chat = (bz_ChatEventData*)eventData;

			bz_PlayerRecord* speaker = bz_getPlayerByIndex ( chat->from );
			if(speaker != NULL){
			std::string message = chat->message.c_str(); //um... yeah
			std::string player = speaker->callsign.c_str();
			int team = speaker->team;
			std::string colorcode;

			if(message.substr(0,1) != "/"){//no slash commands
			if (team == 0)
				colorcode = "\00307"; // rogue
				
			else if(team == 1)
				colorcode = "\00304"; // red team
				
			else if(team == 2)
				colorcode = "\00303"; // green team
				
			else if(team == 3)
				colorcode = "\00302"; // blue team
				
			else if(team == 4)
				colorcode = "\00306"; // purple team

			else if(team == 5)
				colorcode = "\00310"; // observer

			else if(team == 6)
				colorcode = "\00314"; // rabbit

			if(message != "bzadminping"){// if message is not a bzadminping
				std::string subtotal = colorcode + player + ":  " + "\017" + message;
				std::string total = "PRIVMSG " + channel + " :" + subtotal + "\r\n";
				write(fd, total.c_str(), strlen(total.c_str()));
				bz_debugMessage(1,total.c_str());
				break;
			}

			}
			}
		}
		case bz_ePlayerJoinEvent:
			{
				
				bz_PlayerJoinPartEventData *join = (bz_PlayerJoinPartEventData*)eventData;
				bz_PlayerRecord* joiner = bz_getPlayerByIndex ( join->playerID );
				if(joiner != NULL){
				std::string callsign = joiner->callsign.c_str();
				std::string ip = joiner->ipAddress.c_str();
				int team = joiner->team;
				std::string colorcode;
				std::string player_team;
				
				if (team == 0){
				colorcode = "\00307"; // rogue
				player_team = "rogue";
				}
				else if(team == 1){
				colorcode = "\00304"; // red team
				player_team = "red player";
				}
				else if(team == 2){
				colorcode = "\00303"; // green team
				player_team = "green player";
				}
				else if(team == 3){
				colorcode = "\00302"; // blue team
				player_team = "blue player";
				}
				else if(team == 4){
				colorcode = "\00306"; // purple team
				player_team = "purple player";
				}
				else if(team == 5){
				colorcode = "\00310"; // observer
				player_team = "observer";
				}
				else if(team == 6){
				colorcode = "\00314"; // rabbit
				player_team = "rabbit";
				}
				else{
				colorcode = "\017";
			}
				if (callsign != ""){//send it only it is not a list server ping
					std::string subtotal = colorcode + callsign + "\017" + " joined as a " + player_team + " from " + ip;  
					
					if(player_team == "observer"){
						subtotal = colorcode + callsign + "\017" + " joined as a " + player_team + " from " + ip;  
					}

					std::string total = "PRIVMSG " + channel + " :" + subtotal + "\r\n";
					write(fd, total.c_str(), strlen(total.c_str()));
					bz_debugMessage(1,total.c_str());
					break;
				}
				}
			}

		case bz_ePlayerPartEvent:
			{
				
				bz_PlayerJoinPartEventData *part = (bz_PlayerJoinPartEventData*)eventData;
				if(part != NULL){
				std::string callsign = part->callsign.c_str();
				int team = part->team;
				std::string colorcode;
				
			if (team == 0)
				colorcode = "\00307"; // rogue
				
			else if(team == 1)
				colorcode = "\00304"; // red team
				
			else if(team == 2)
				colorcode = "\00303"; // green team
				
			else if(team == 3)
				colorcode = "\00302"; // blue team
				
			else if(team == 4)
				colorcode = "\00306"; // purple team

			else if(team == 5)
				colorcode = "\00310"; // observer

			else if(team == 6)
				colorcode = "\00314"; // rabbit
				
			else
				colorcode = "\017";

			if (callsign != ""){//send it only it is not a list server ping
				std::string subtotal = colorcode + callsign + "\017" + " left the game";  
				std::string total = "PRIVMSG " + channel + " :" + subtotal + "\r\n";
				write(fd, total.c_str(), strlen(total.c_str()));
				bz_debugMessage(1,total.c_str());
				break;
			}
			}
			}
		default:
			break;

		

	}
}

void* respondPing(void* t){
while (1) {
		char recv_buf[1024];
		int r_len = read(fd, recv_buf, 1024);
		recv_buf[r_len] = '\0';
		
		std::string data = recv_buf;
		
		if (data.find ("PRIVMSG", 0) != std::string::npos){ // send only privmsgs
		std::string::size_type pos = data.find (":", 0);
		std::string::size_type endpos = data.find("!", 0);
		std::string username = data.substr(pos+1, endpos-1);

		pos = data.find (":", 1); // find the other :
		endpos = data.size();
		std::string message = data.substr(pos+1, endpos);

		std::string total = username + ": " + message;
		
		bz_sendTextMessage (BZ_SERVER, BZ_ALLUSERS, total.c_str());
		}

		if(data.substr(0,4) == "PING"){
		std::string ping = data;
		std::string pongdata = ping.substr(5 ,ping.size());
		
		std::string pong = "PONG " + pongdata + "\n";
		
		write(fd, pong.c_str(), pong.size());
		
		}
}

}
